/*
Copyright (c) 2019 Steven Spangler <132@ikl.sh>
This file is part of liteshort by 132ikl
This software is license under the MIT license. It should be included in your copy of this software.
A copy of the MIT license can be obtained at https://mit-license.org/
*/

input {
  margin: auto;
}

div.form {
  margin-top: 5%;
  text-align: center;
}

div.success {
  display: inline-block;
  font-family: Open Sans;
  border-radius: 2vh;
  padding: 2vh;
  color: #62ad2c;
  background-color: #E9FFD9;
  border: 1px solid #62ad2c;
}


div.error {
  display: inline-block;
  font-family: Open Sans;
  border-radius: 2vh;
  padding: 2vh;
  color: #a86464;
  background-color: #FCE9E9;
  border: 1px solid #a86464;
}

body {
  text-align: center
}

h2 {
  font-weight: normal;
  font-family: Open Sans;
}

div.success > a:link {
  color: #62ad2c;
}

div.success > a:visited {
  color: #507c52;
}

div.github {
  position: absolute;
  transition: opacity .25s ease-out;
  -moz-transition: opacity .25s ease-out;
  -webkit-transition: opacity .25s ease-out;
  -o-transition: opacity .25s ease-out;
  opacity: 0.3;
  right: 2.5vw;
  bottom: 15vh;
  width: 50px;
  height: 10px;
}

div.github:hover {
 opacity: 1; 
}